"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DOMRenderer = void 0;
var tslib_1 = require("tslib");
require("scheduler-polyfill");
var web_streams_polyfill_1 = require("web-streams-polyfill");
var web_utility_1 = require("web-utility");
var VDOM_1 = require("./VDOM");
var DOMRenderer = /** @class */ (function () {
    function DOMRenderer() {
        var _this = this;
        this.eventPattern = /^on[A-Z]/;
        this.ariaPattern = /^aira[A-Z]/;
        this.document = globalThis.document;
        this.treeCache = new WeakMap();
        this.signalCache = new WeakMap();
        this.keyOf = function (_a, index) {
            var key = _a.key, text = _a.text, props = _a.props, selector = _a.selector;
            return (key === null || key === void 0 ? void 0 : key.toString()) || (props === null || props === void 0 ? void 0 : props.id) || (text || selector || '') + index;
        };
        this.vNodeOf = function (list, key) {
            return list.find(function (vNode, index) { return "".concat(_this.keyOf(vNode, index)) === String(key); });
        };
        this.propsKeyOf = function (key) {
            return key.startsWith('aria-')
                ? (0, web_utility_1.toCamelCase)(key)
                : _this.eventPattern.test(key)
                    ? key.toLowerCase()
                    : key;
        };
        this.attrsNameOf = function (key) { return VDOM_1.VNode.propsMap[key] || key; };
        this.removeProperty = function (node, key) {
            return _this.eventPattern.test(key)
                ? (node[key.toLowerCase()] = null)
                : node.removeAttribute(_this.ariaPattern.test(key) ? (0, web_utility_1.toHyphenCase)(key) : _this.attrsNameOf(key));
        };
        this.setProperty = function (node, key, value) {
            var isXML = (0, web_utility_1.templateOf)(node.tagName) && (0, web_utility_1.elementTypeOf)(node.tagName) === 'xml';
            if (isXML || key.includes('-'))
                node.setAttribute(_this.attrsNameOf(key), value);
            else
                try {
                    var name_1 = _this.propsKeyOf(key);
                    if (_this.eventPattern.test(key) && !(name_1 in node))
                        _this.handleCustomEvent(node, name_1.slice(2));
                    node[name_1] = value;
                }
                catch (_a) {
                    node.setAttribute(_this.attrsNameOf(key), value);
                }
        };
    }
    DOMRenderer.prototype.updateProps = function (node, oldProps, newProps, onDelete, onAdd) {
        var e_1, _a, e_2, _b;
        if (oldProps === void 0) { oldProps = {}; }
        if (newProps === void 0) { newProps = {}; }
        var group = (0, web_utility_1.diffKeys)(Object.keys(oldProps), Object.keys(newProps)).group;
        try {
            for (var _c = tslib_1.__values(group[web_utility_1.DiffStatus.Old] || []), _d = _c.next(); !_d.done; _d = _c.next()) {
                var _e = tslib_1.__read(_d.value, 1), key = _e[0];
                onDelete === null || onDelete === void 0 ? void 0 : onDelete(node, key);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
        try {
            for (var _f = tslib_1.__values(tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read((group[web_utility_1.DiffStatus.Same] || [])), false), tslib_1.__read((group[web_utility_1.DiffStatus.New] || [])), false)), _g = _f.next(); !_g.done; _g = _f.next()) {
                var _h = tslib_1.__read(_g.value, 1), key = _h[0];
                if (oldProps[key] !== newProps[key])
                    if (onAdd instanceof Function)
                        onAdd(node, key, newProps[key]);
                    else
                        Reflect.set(node, key, newProps[key]);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_g && !_g.done && (_b = _f.return)) _b.call(_f);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    DOMRenderer.prototype.deleteNode = function (_a) {
        var ref = _a.ref, node = _a.node, children = _a.children;
        if (node instanceof DocumentFragment)
            children === null || children === void 0 ? void 0 : children.forEach(this.deleteNode);
        else if (node) {
            node.remove();
            ref === null || ref === void 0 ? void 0 : ref();
        }
    };
    DOMRenderer.prototype.commitChild = function (root, node, index) {
        if (index === void 0) { index = 0; }
        var targetNode = root.childNodes[index];
        if (targetNode === node)
            return;
        if (!targetNode)
            root.append(node);
        else
            targetNode.before(node);
    };
    DOMRenderer.prototype.diffVChildren = function (oldVNode, newVNode) {
        var _a, map, group, deletingGroup, _b, _c, _d, index, newVChild, key, oldVChild, e_3_1, _e, _f, _g, _i, selector, _h, _j, oldVNode_1, e_4_1;
        var e_3, _k, e_4, _l;
        var _this = this;
        var _m;
        return tslib_1.__generator(this, function (_o) {
            switch (_o.label) {
                case 0:
                    newVNode.children = newVNode.children.map(function (vNode) { return new VDOM_1.VNode(vNode); });
                    _a = (0, web_utility_1.diffKeys)(oldVNode.children.map(this.keyOf), newVNode.children.map(this.keyOf)), map = _a.map, group = _a.group;
                    deletingGroup = group[web_utility_1.DiffStatus.Old] &&
                        (0, web_utility_1.groupBy)(group[web_utility_1.DiffStatus.Old].map(function (_a) {
                            var _b = tslib_1.__read(_a, 1), key = _b[0];
                            return _this.vNodeOf(oldVNode.children, key);
                        }), function (_a) {
                            var selector = _a.selector;
                            return selector + '';
                        });
                    _o.label = 1;
                case 1:
                    _o.trys.push([1, 7, 8, 9]);
                    _b = tslib_1.__values(newVNode.children.entries()), _c = _b.next();
                    _o.label = 2;
                case 2:
                    if (!!_c.done) return [3 /*break*/, 6];
                    _d = tslib_1.__read(_c.value, 2), index = _d[0], newVChild = _d[1];
                    key = this.keyOf(newVChild, index);
                    oldVChild = map[key] === web_utility_1.DiffStatus.Same
                        ? this.vNodeOf(oldVNode.children, key)
                        : (_m = deletingGroup === null || deletingGroup === void 0 ? void 0 : deletingGroup[newVChild.selector]) === null || _m === void 0 ? void 0 : _m.shift();
                    return [4 /*yield*/, { index: index, oldVNode: oldVChild, newVNode: newVChild }];
                case 3:
                    _o.sent();
                    if (!((oldVChild === null || oldVChild === void 0 ? void 0 : oldVChild.children[0]) || newVChild.children[0])) return [3 /*break*/, 5];
                    oldVChild || (oldVChild = new VDOM_1.VNode(tslib_1.__assign(tslib_1.__assign({}, newVChild), { children: [] })));
                    return [5 /*yield**/, tslib_1.__values(this.diffVChildren(oldVChild, newVChild))];
                case 4:
                    _o.sent();
                    _o.label = 5;
                case 5:
                    _c = _b.next();
                    return [3 /*break*/, 2];
                case 6: return [3 /*break*/, 9];
                case 7:
                    e_3_1 = _o.sent();
                    e_3 = { error: e_3_1 };
                    return [3 /*break*/, 9];
                case 8:
                    try {
                        if (_c && !_c.done && (_k = _b.return)) _k.call(_b);
                    }
                    finally { if (e_3) throw e_3.error; }
                    return [7 /*endfinally*/];
                case 9:
                    _e = deletingGroup;
                    _f = [];
                    for (_g in _e)
                        _f.push(_g);
                    _i = 0;
                    _o.label = 10;
                case 10:
                    if (!(_i < _f.length)) return [3 /*break*/, 19];
                    _g = _f[_i];
                    if (!(_g in _e)) return [3 /*break*/, 18];
                    selector = _g;
                    _o.label = 11;
                case 11:
                    _o.trys.push([11, 16, 17, 18]);
                    _h = (e_4 = void 0, tslib_1.__values(deletingGroup[selector])), _j = _h.next();
                    _o.label = 12;
                case 12:
                    if (!!_j.done) return [3 /*break*/, 15];
                    oldVNode_1 = _j.value;
                    return [4 /*yield*/, { oldVNode: oldVNode_1 }];
                case 13:
                    _o.sent();
                    _o.label = 14;
                case 14:
                    _j = _h.next();
                    return [3 /*break*/, 12];
                case 15: return [3 /*break*/, 18];
                case 16:
                    e_4_1 = _o.sent();
                    e_4 = { error: e_4_1 };
                    return [3 /*break*/, 18];
                case 17:
                    try {
                        if (_j && !_j.done && (_l = _h.return)) _l.call(_h);
                    }
                    finally { if (e_4) throw e_4.error; }
                    return [7 /*endfinally*/];
                case 18:
                    _i++;
                    return [3 /*break*/, 10];
                case 19: return [2 /*return*/];
            }
        });
    };
    DOMRenderer.prototype.handleCustomEvent = function (node, event) {
        var handler;
        Object.defineProperty(node, "on".concat(event), {
            set: function (value) {
                if (handler)
                    node.removeEventListener(event, handler);
                node.addEventListener(event, (handler = value));
            },
            get: function () { return handler; }
        });
    };
    DOMRenderer.prototype.patchNode = function (oldVNode, newVNode) {
        this.updateProps(oldVNode.node, oldVNode.props, newVNode.props, this.removeProperty, this.setProperty);
        this.updateProps(oldVNode.node.style, oldVNode.style, newVNode.style, function (style, key) { return style.removeProperty((0, web_utility_1.toHyphenCase)(key)); }, function (style, key, value) { return style.setProperty((0, web_utility_1.toHyphenCase)(key), value); });
        newVNode.node || (newVNode.node = oldVNode.node);
        return newVNode;
    };
    DOMRenderer.prototype.generateDOM = function (oldVRoot, newVRoot) {
        var _a, _b, _c, index, oldVNode, newVNode, inserting, tagName, node, parent_1, e_5_1;
        var e_5, _d;
        var _e;
        return tslib_1.__generator(this, function (_f) {
            switch (_f.label) {
                case 0:
                    if (VDOM_1.VNode.isFragment(newVRoot))
                        newVRoot = new VDOM_1.VNode(tslib_1.__assign(tslib_1.__assign({}, oldVRoot), { children: newVRoot.children }));
                    return [4 /*yield*/, this.patchNode(oldVRoot, newVRoot)];
                case 1:
                    _f.sent();
                    _f.label = 2;
                case 2:
                    _f.trys.push([2, 7, 8, 9]);
                    _a = tslib_1.__values(this.diffVChildren(oldVRoot, newVRoot)), _b = _a.next();
                    _f.label = 3;
                case 3:
                    if (!!_b.done) return [3 /*break*/, 6];
                    _c = _b.value, index = _c.index, oldVNode = _c.oldVNode, newVNode = _c.newVNode;
                    if (!newVNode) {
                        this.deleteNode(oldVNode);
                        return [3 /*break*/, 5];
                    }
                    inserting = !oldVNode;
                    if (oldVNode)
                        newVNode.node = oldVNode.node;
                    else {
                        newVNode.createDOM(this.document);
                        tagName = newVNode.tagName, node = newVNode.node, parent_1 = newVNode.parent;
                        oldVNode = new VDOM_1.VNode({ tagName: tagName, node: node, parent: parent_1 });
                    }
                    if (newVNode.text)
                        oldVNode.node.nodeValue = newVNode.text;
                    else if (!VDOM_1.VNode.isFragment(newVNode))
                        this.patchNode(oldVNode, newVNode);
                    if (oldVNode.parent) {
                        this.commitChild(oldVNode.parent.node, newVNode.node, index);
                        if (inserting)
                            (_e = newVNode.ref) === null || _e === void 0 ? void 0 : _e.call(newVNode, newVNode.node);
                    }
                    return [4 /*yield*/, newVNode];
                case 4:
                    _f.sent();
                    _f.label = 5;
                case 5:
                    _b = _a.next();
                    return [3 /*break*/, 3];
                case 6: return [3 /*break*/, 9];
                case 7:
                    e_5_1 = _f.sent();
                    e_5 = { error: e_5_1 };
                    return [3 /*break*/, 9];
                case 8:
                    try {
                        if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                    }
                    finally { if (e_5) throw e_5.error; }
                    return [7 /*endfinally*/];
                case 9: return [2 /*return*/];
            }
        });
    };
    DOMRenderer.prototype.patch = function (oldVRoot, newVRoot) {
        var e_6, _a;
        var count = 0;
        try {
            for (var _b = tslib_1.__values(this.generateDOM(oldVRoot, newVRoot)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var newVNode = _c.value;
                if (++count === 1)
                    newVRoot = newVNode;
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_6) throw e_6.error; }
        }
        return newVRoot;
    };
    DOMRenderer.prototype.patchAsync = function (oldVRoot, newVRoot) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var oldController, controller, count, _a, _b, newVNode, e_7_1;
            var e_7, _c;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        oldController = this.signalCache.get(oldVRoot.node);
                        if (oldController) {
                            oldController.abort();
                            oldVRoot = VDOM_1.VNode.fromDOM(oldVRoot.node);
                        }
                        controller = new AbortController();
                        this.signalCache.set(oldVRoot.node, controller);
                        count = 0;
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        _a = tslib_1.__values(this.generateDOM(oldVRoot, newVRoot)), _b = _a.next();
                        _d.label = 2;
                    case 2:
                        if (!!_b.done) return [3 /*break*/, 5];
                        newVNode = _b.value;
                        if (++count === 1)
                            newVRoot = newVNode;
                        return [4 /*yield*/, scheduler.yield()];
                    case 3:
                        _d.sent();
                        if (controller.signal.aborted) {
                            this.signalCache.delete(oldVRoot.node);
                            controller.signal.throwIfAborted();
                        }
                        _d.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_7_1 = _d.sent();
                        e_7 = { error: e_7_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_7) throw e_7.error; }
                        return [7 /*endfinally*/];
                    case 8:
                        this.signalCache.delete(oldVRoot.node);
                        return [2 /*return*/, newVRoot];
                }
            });
        });
    };
    DOMRenderer.prototype.render = function (vNode, node, mode) {
        var _this = this;
        var _a;
        if (node === void 0) { node = (_a = globalThis.document) === null || _a === void 0 ? void 0 : _a.body; }
        if (mode === void 0) { mode = 'sync'; }
        this.document = node.ownerDocument;
        var root = this.treeCache.get(node) || VDOM_1.VNode.fromDOM(node);
        var done = function (root) { return _this.treeCache.set(node, root) && root; };
        return mode === 'sync'
            ? done(this.patch(root, new VDOM_1.VNode(tslib_1.__assign(tslib_1.__assign({}, root), { children: [vNode] }))))
            : this.patchAsync(root, new VDOM_1.VNode(tslib_1.__assign(tslib_1.__assign({}, root), { children: [vNode] }))).then(done);
    };
    DOMRenderer.prototype.renderToStaticMarkup = function (tree) {
        return tslib_1.__spreadArray([], tslib_1.__read(tree.generateXML()), false).join('');
    };
    DOMRenderer.prototype.renderToReadableStream = function (tree) {
        return web_streams_polyfill_1.ReadableStream.from(tree.generateXML());
    };
    return DOMRenderer;
}());
exports.DOMRenderer = DOMRenderer;
